#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Message::Correlator::Engine module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 7;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Message::Correlator::Engine';

        use_ok('Tester');
        use_ok('Mentat::Message::Uniter');
        use_ok('Mentat::Message::Correlator::Context');
        use_ok('Mentat::Message::Correlator::Context::Queue::Memory');
        use_ok('Mentat::Message::Factory');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Mentat::Message::Correlator::Engine::VERSION

### Attempting to create Mentat::Message::Correlator::Engine instance...
my $instance = $TESTED_CLASS->new('../../conf/filter-grammar.conf', './conf/traggregation-rules.conf');
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

my $MESSAGE_RULES1 = [
    ['Alert/@messageid','abc123456789'],
    ['Alert/Analyzer/@analyzerid','hq-dmz-analyzer0'],
    ['Alert/Analyzer/Node/@category','dns'],
    ['Alert/Analyzer/Node/location','Headquarters DMZ Network'],
    ['Alert/Analyzer/Node/name','analyzer01.example.com'],
    ['Alert/CreateTime/@ntpstamp','2000-03-09T18:01:25-05:00'],
    ['Alert/Source/@ident','a1b2c3d4'],
    ['Alert/Source/Node/@ident','a1b2c3d4-001'],
    ['Alert/Source/Node/@category','dns'],
    ['Alert/Source/Node/name','badguy.example.net'],
    ['Alert/Source/Node/Address/address','50.50.50.50'],
    ['Alert/Classification/@text','Teardrop detected'],
    ['Alert/Classification/Reference/@origin','bugtraqid'],
    ['Alert/Classification/Reference/name','124'],
    ['Alert/Classification/Reference/url','http://www.securityfocus.com/bid/124'],
];
my $MESSAGE_RULES2 = [
    ['Alert/@messageid','abc123456789'],
    ['Alert/Analyzer/@analyzerid','hq-dmz-analyzer0'],
    ['Alert/Analyzer/Node/@category','dns'],
    ['Alert/Analyzer/Node/location','Headquarters DMZ Network'],
    ['Alert/Analyzer/Node/name','analyzer01.example.com'],
    ['Alert/CreateTime/@ntpstamp','2000-03-09T10:01:25-05:00'],
    ['Alert/Source/@ident','a1b2c3d4'],
    ['Alert/Source/Node/@ident','a1b2c3d4-001'],
    ['Alert/Source/Node/@category','dns'],
    ['Alert/Source/Node/name','badguy.example.net'],
    ['Alert/Source/Node/Address/address','50.50.50.50'],
    ['Alert/Classification/@text','Teardrop detected'],
    ['Alert/Classification/Reference/@origin','bugtraqid'],
    ['Alert/Classification/Reference/name','124'],
    ['Alert/Classification/Reference/url','http://www.securityfocus.com/bid/124'],
];
my $MESSAGE_RULES3 = [
    ['Alert/@messageid','abc123456789'],
    ['Alert/Analyzer/@analyzerid','hq-dmz-analyzer0'],
    ['Alert/Analyzer/Node/@category','dns'],
    ['Alert/Analyzer/Node/location','Headquarters DMZ Network'],
    ['Alert/Analyzer/Node/name','analyzer01.example.com'],
    ['Alert/CreateTime/@ntpstamp','2000-03-09T00:01:25-05:00'],
    ['Alert/Source/@ident','a1b2c3d4'],
    ['Alert/Source/Node/@ident','a1b2c3d4-001'],
    ['Alert/Source/Node/@category','dns'],
    ['Alert/Source/Node/name','badguy.example.net'],
    ['Alert/Source/Node/Address/address','50.50.50.50'],
    ['Alert/Classification/@text','Teardrop detected'],
    ['Alert/Classification/Reference/@origin','bugtraqid'],
    ['Alert/Classification/Reference/name','124'],
    ['Alert/Classification/Reference/url','http://www.securityfocus.com/bid/124'],
];

my $msg1 = Mentat::Message::Factory->from_rules($MESSAGE_RULES1, 'idea');
my $msg2 = Mentat::Message::Factory->from_rules($MESSAGE_RULES2, 'idea');
my $msg3 = Mentat::Message::Factory->from_rules($MESSAGE_RULES3, 'idea');

#diag(Dumper($msg1)) if Tester::D(2);

diag("C1: " . Dumper($instance->correlate($msg1))) if Tester::D();
diag("Q1: " . Dumper($instance->{Q_CONTEXTS})) if Tester::D();
diag("C2: " . Dumper($instance->correlate($msg2))) if Tester::D();
diag("Q2: " . Dumper($instance->{Q_CONTEXTS})) if Tester::D();
diag("C3: " . Dumper($instance->correlate($msg3))) if Tester::D();
diag("Q3: " . Dumper($instance->{Q_CONTEXTS})) if Tester::D();

sleep 10 if Tester::D();

diag("CC: " . $instance->check_contexts()) if Tester::D();
